主頁 > 企業開發 > 執行緒數增加但對運行時沒有影響

執行緒數增加但對運行時沒有影響

2022-10-19 09:42:21 企業開發

我試圖在 CUDA C 中實作 alpha 影像混合演算法。我的代碼沒有錯誤。它編譯得很好。根據執行緒邏輯,如果我以增加的執行緒數運行代碼,則應該減少運行時。在我的代碼中,我得到了一種奇怪的運行時模式。當我用 1 個執行緒運行代碼時,運行時為 8.060539 e-01 秒,當我用 4 個執行緒運行代碼時,我得到運行時 7.579031 e-01 秒,當它運行 8 個執行緒時,運行時為 7.810102e-01,并且對于 256 個執行緒,運行時為 7.875319e-01。

這是我的代碼:

#include <stdio.h>
#include <stdlib.h>
#include "timer.h"
#define STB_IMAGE_IMPLEMENTATION
#include "stb_image.h"
#define STB_IMAGE_WRITE_IMPLEMENTATION
#include "stb_image_write.h"

__global__ void image_blend(unsigned char *Pout, unsigned char *pin1, unsigned char *pin2, int width, int height, int channels, float alpha){

        int col = threadIdx.x   blockIdx.x*blockDim.x;
        int row = threadIdx.y   blockIdx.y*blockDim.y;


        if(col<width && row<height){
                    size_t img_size = width * height * channels;

                        if (Pout != NULL)
                                {
                                    for (size_t i = 0; i < img_size; i  )
                                    {
                                        Pout[i] = ((1.0 - alpha) * pin1[i]   alpha * pin2[i]);
                                    }
                                }

        }

}

int main(int argc, char* argv[]){

        int thread_count;
        double start, finish;
        float alpha;
        int width, height, channels;
        unsigned char *new_img;
        thread_count = strtol(argv[1], NULL, 10);
        printf("Enter the value for alpha:");
        scanf("%f", &alpha);
        unsigned char *apple = stbi_load("apple.jpg", &width, &height, &channels, 0);
        unsigned char *orange = stbi_load("orange.jpg", &width, &height, &channels, 0);
        size_t img_size = width * height * channels;
        //unsigned char *new_img = malloc(img_size);
        cudaMallocManaged(&new_img,img_size*sizeof(unsigned char));
        cudaMallocManaged(&apple,img_size* sizeof(unsigned char));
        cudaMallocManaged(&orange, img_size*sizeof(unsigned char));
        GET_TIME(start);
        image_blend<<<1,16,thread_count>>>(new_img,apple, orange, width, height, channels,alpha);
        cudaDeviceSynchronize();
        GET_TIME(finish);
        stbi_write_jpg("new_image.jpg", width, height, channels, new_img, 100);
        cudaFree(new_img);
        cudaFree(apple);
        cudaFree(orange);
        printf("\n Elapsed time for cuda = %e seconds\n", finish-start);

}

在運行時得到一個奇怪的模式后,我對代碼的實作有點懷疑。即使我的代碼沒有錯誤,任何人都可以讓我知道為什么我得到那些運行時。

uj5u.com熱心網友回復:

讓我們從這里開始:

image_blend<<<1,16,thread_count>>>(new_img,apple, orange, width, height, channels,alpha);

顯然您不了解內核啟動語法:

<<<1,16,thread_count>>>

第一個數字 (1) 是要啟動的塊數。第二個數字 (16) 是每個塊的執行緒數。第三個數字 ( thread_count) 是動態分配的共享記憶體的大小(以位元組為單位)。

因此,我們的第一個觀察結果是,盡管您聲稱更改了執行緒數,但您沒有。您正在更改動態分配的共享記憶體的位元組數。由于您的內核代碼不使用共享記憶體,因此這是一個完全沒有意義的變數。

讓我們也觀察一下你的內核代碼:

                                for (size_t i = 0; i < img_size; i  )
                                {
                                    Pout[i] = ((1.0 - alpha) * pin1[i]   alpha * pin2[i]);
                                }

對于通過 if 測驗的每個執行緒,這些執行緒中的每一個都將執行整個 for 回圈并處理整個影像。這不是撰寫 CUDA 內核的一般想法。一般的想法是分解作業,以便每個執行緒完成一部分作業,而不是整個活動。

這些是非常基本的觀察結果。如果您利用對 CUDA 的有序介紹(例如此處),您可以超越其中一些基本概念。

我們還可以指出,您的內核名義上期望 2D 啟動,而您沒有提供一個,也許還有許多其他觀察結果。您缺少的另一個重要概念是您不能這樣做:

    unsigned char *apple = stbi_load("apple.jpg", &width, &height, &channels, 0);
    ...
    cudaMallocManaged(&apple,img_size* sizeof(unsigned char));

并期望從中獲得任何明智的結果。如果您想了解資料如何從主機分配移動到設備,請研究幾乎所有 CUDA 示例代碼,例如vectorAdd. 使用托管分配不允許您像正在做的那樣覆寫指標并從中獲得任何有用的東西。

我將提供一個示例,說明如何去做我認為您建議的事情,而不提供有關 CUDA 的完整教程。舉個例子,我將跳過 STB 影像加載例程。要了解您在此處嘗試執行的作業,實際的影像內容并不重要。

這是一個影像處理內核 (1D) 的示例,它將:

  1. 處理整個影像,只處理一次
  2. 粗略地說,隨著執行緒數的增加,使用更少的時間。

你還沒有提供你的計時器例程/代碼,所以我會提供我自己的:

$ cat t2130.cu
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/time.h>
#define USECPSEC 1000000ULL

unsigned long long dtime_usec(unsigned long long start=0){

  timeval tv;
  gettimeofday(&tv, 0);
  return ((tv.tv_sec*USECPSEC) tv.tv_usec)-start;
}

unsigned char *i_load(int w, int h, int c, int init){
  unsigned char *res = new unsigned char[w*h*c];
  for (int i = 0; i < w*h*c; i  ) res[i] = init;
  return res;
}

__global__ void image_blend(unsigned char *Pout, unsigned char *pin1, unsigned char *pin2, int width, int height, int channels, float alpha){

  if (Pout != NULL)
    {
    size_t img_size = width * height * channels;
    for (size_t i = blockIdx.x*blockDim.x threadIdx.x; i < img_size; i =gridDim.x*blockDim.x) // grid-stride loop
      {
        Pout[i] = ((1.0 - alpha) * pin1[i]   alpha * pin2[i]);
      }
    }
}

int main(int argc, char* argv[]){

        int threads_per_block = 64;
        unsigned long long dt;
        float alpha;
        int width = 1920;
        int height = 1080;
        int channels = 3;
        size_t img_size = width * height * channels;
        int thread_count = img_size;
        if (argc > 1) thread_count = atoi(argv[1]);
        unsigned char *new_img, *m_apple, *m_orange;
        printf("Enter the value for alpha:");
        scanf("%f", &alpha);
        unsigned char *apple = i_load(width, height, channels, 10);
        unsigned char *orange = i_load(width, height, channels, 70);
        //unsigned char *new_img = malloc(img_size);
        cudaMallocManaged(&new_img,img_size*sizeof(unsigned char));
        cudaMallocManaged(&m_apple,img_size* sizeof(unsigned char));
        cudaMallocManaged(&m_orange, img_size*sizeof(unsigned char));
        memcpy(m_apple, apple, img_size);
        memcpy(m_orange, orange, img_size);
        int blocks;
        if (thread_count < threads_per_block) {threads_per_block = thread_count; blocks = 1;}
        else {blocks = thread_count/threads_per_block;}
        printf("running with %d blocks of %d threads\n", blocks, threads_per_block);
        dt = dtime_usec(0);
        image_blend<<<blocks, threads_per_block>>>(new_img,m_apple, m_orange, width, height, channels,alpha);
        cudaDeviceSynchronize();
        dt = dtime_usec(dt);
        cudaError_t err = cudaGetLastError();
        if (err != cudaSuccess) printf("CUDA Error: %s\n", cudaGetErrorString(err));
        else printf("\n Elapsed time for cuda = %e seconds\n", dt/(float)USECPSEC);
        cudaFree(new_img);
        cudaFree(m_apple);
        cudaFree(m_orange);
}
$ nvcc -o t2130 t2130.cu
$ ./t2130 1
Enter the value for alpha:0.2
running with 1 blocks of 1 threads

 Elapsed time for cuda = 5.737880e-01 seconds
$ ./t2130 2
Enter the value for alpha:0.2
running with 1 blocks of 2 threads

 Elapsed time for cuda = 3.230150e-01 seconds
$ ./t2130 32
Enter the value for alpha:0.2
running with 1 blocks of 32 threads

 Elapsed time for cuda = 4.865200e-02 seconds
$ ./t2130 64
Enter the value for alpha:0.2
running with 1 blocks of 64 threads

 Elapsed time for cuda = 2.623300e-02 seconds
$ ./t2130 128
Enter the value for alpha:0.2
running with 2 blocks of 64 threads

 Elapsed time for cuda = 1.546000e-02 seconds
$ ./t2130
Enter the value for alpha:0.2
running with 97200 blocks of 64 threads

 Elapsed time for cuda = 5.809000e-03 seconds
$

(CentOS 7、CUDA 11.4、V100)

允許內核完成所有作業(僅一次)同時有效地使用“任意”數量的執行緒的關鍵方法是grid-stride loop

轉載請註明出處,本文鏈接:https://www.uj5u.com/qiye/517663.html

標籤:图像处理库达字母混合

上一篇:JS執行機制

下一篇:使用Pillow使多行文本居中而不脫離影像

標籤雲
其他(157675) Python(38076) JavaScript(25376) Java(17977) C(15215) 區塊鏈(8255) C#(7972) AI(7469) 爪哇(7425) MySQL(7132) html(6777) 基礎類(6313) sql(6102) 熊猫(6058) PHP(5869) 数组(5741) R(5409) Linux(5327) 反应(5209) 腳本語言(PerlPython)(5129) 非技術區(4971) Android(4554) 数据框(4311) css(4259) 节点.js(4032) C語言(3288) json(3245) 列表(3129) 扑(3119) C++語言(3117) 安卓(2998) 打字稿(2995) VBA(2789) Java相關(2746) 疑難問題(2699) 细绳(2522) 單片機工控(2479) iOS(2429) ASP.NET(2402) MongoDB(2323) 麻木的(2285) 正则表达式(2254) 字典(2211) 循环(2198) 迅速(2185) 擅长(2169) 镖(2155) 功能(1967) .NET技术(1958) Web開發(1951) python-3.x(1918) HtmlCss(1915) 弹簧靴(1913) C++(1909) xml(1889) PostgreSQL(1872) .NETCore(1853) 谷歌表格(1846) Unity3D(1843) for循环(1842)

熱門瀏覽
  • IEEE1588PTP在數字化變電站時鐘同步方面的應用

    IEEE1588ptp在數字化變電站時鐘同步方面的應用 京準電子科技官微——ahjzsz 一、電力系統時間同步基本概況 隨著對IEC 61850標準研究的不斷深入,國內外學者提出基于IEC61850通信標準體系建設數字化變電站的發展思路。數字化變電站與常規變電站的顯著區別在于程序層傳統的電流/電壓互 ......

    uj5u.com 2020-09-10 03:51:52 more
  • HTTP request smuggling CL.TE

    CL.TE 簡介 前端通過Content-Length處理請求,通過反向代理或者負載均衡將請求轉發到后端,后端Transfer-Encoding優先級較高,以TE處理請求造成安全問題。 檢測 發送如下資料包 POST / HTTP/1.1 Host: ac391f7e1e9af821806e890 ......

    uj5u.com 2020-09-10 03:52:11 more
  • 網路滲透資料大全單——漏洞庫篇

    網路滲透資料大全單——漏洞庫篇漏洞庫 NVD ——美國國家漏洞庫 →http://nvd.nist.gov/。 CERT ——美國國家應急回應中心 →https://www.us-cert.gov/ OSVDB ——開源漏洞庫 →http://osvdb.org Bugtraq ——賽門鐵克 →ht ......

    uj5u.com 2020-09-10 03:52:15 more
  • 京準講述NTP時鐘服務器應用及原理

    京準講述NTP時鐘服務器應用及原理京準講述NTP時鐘服務器應用及原理 安徽京準電子科技官微——ahjzsz 北斗授時原理 授時是指接識訓通過某種方式獲得本地時間與北斗標準時間的鐘差,然后調整本地時鐘使時差控制在一定的精度范圍內。 衛星導航系統通常由三部分組成:導航授時衛星、地面檢測校正維護系統和用戶 ......

    uj5u.com 2020-09-10 03:52:25 more
  • 利用北斗衛星系統設計NTP網路時間服務器

    利用北斗衛星系統設計NTP網路時間服務器 利用北斗衛星系統設計NTP網路時間服務器 安徽京準電子科技官微——ahjzsz 概述 NTP網路時間服務器是一款支持NTP和SNTP網路時間同步協議,高精度、大容量、高品質的高科技時鐘產品。 NTP網路時間服務器設備采用冗余架構設計,高精度時鐘直接來源于北斗 ......

    uj5u.com 2020-09-10 03:52:35 more
  • 詳細解讀電力系統各種對時方式

    詳細解讀電力系統各種對時方式 詳細解讀電力系統各種對時方式 安徽京準電子科技官微——ahjzsz,更多資料請添加VX 衛星同步時鐘是我京準公司開發研制的應用衛星授時時技術的標準時間顯示和發送的裝置,該裝置以M國全球定位系統(GLOBAL POSITIONING SYSTEM,縮寫為GPS)或者我國北 ......

    uj5u.com 2020-09-10 03:52:45 more
  • 如何保證外包團隊接入企業內網安全

    不管企業規模的大小,只要企業想省錢,那么企業的某些服務就一定會采用外包的形式,然而看似美好又經濟的策略,其實也有不好的一面。下面我通過安全的角度來聊聊使用外包團的安全隱患問題。 先看看什么服務會使用外包的,最常見的就是話務/客服這種需要大量重復性、無技術性的服務,或者是一些銷售外包、特殊的職能外包等 ......

    uj5u.com 2020-09-10 03:52:57 more
  • PHP漏洞之【整型數字型SQL注入】

    0x01 什么是SQL注入 SQL是一種注入攻擊,通過前端帶入后端資料庫進行惡意的SQL陳述句查詢。 0x02 SQL整型注入原理 SQL注入一般發生在動態網站URL地址里,當然也會發生在其它地發,如登錄框等等也會存在注入,只要是和資料庫打交道的地方都有可能存在。 如這里http://192.168. ......

    uj5u.com 2020-09-10 03:55:40 more
  • [GXYCTF2019]禁止套娃

    git泄露獲取原始碼 使用GET傳參,引數為exp 經過三層過濾執行 第一層過濾偽協議,第二層過濾帶引數的函式,第三層過濾一些函式 preg_replace('/[a-z,_]+\((?R)?\)/', NULL, $_GET['exp'] (?R)參考當前正則運算式,相當于匹配函式里的引數 因此傳遞 ......

    uj5u.com 2020-09-10 03:56:07 more
  • 等保2.0實施流程

    流程 結論 ......

    uj5u.com 2020-09-10 03:56:16 more
最新发布
  • 使用Django Rest framework搭建Blog

    在前面的Blog例子中我們使用的是GraphQL, 雖然GraphQL的使用處于上升趨勢,但是Rest API還是使用的更廣泛一些. 所以還是決定回到傳統的rest api framework上來, Django rest framework的官網上給了一個很好用的QuickStart, 我參考Qu ......

    uj5u.com 2023-04-20 08:17:54 more
  • 記錄-new Date() 我忍你很久了!

    這里給大家分享我在網上總結出來的一些知識,希望對大家有所幫助 大家平時在開發的時候有沒被new Date()折磨過?就是它的諸多怪異的設定讓你每每用的時候,都可能不小心踩坑。造成程式意外出錯,卻一下子找不到問題出處,那叫一個煩透了…… 下面,我就列舉它的“四宗罪”及應用思考 可惡的四宗罪 1. Sa ......

    uj5u.com 2023-04-20 08:17:47 more
  • 使用Vue.js實作文字跑馬燈效果

    實作文字跑馬燈效果,首先用到 substring()截取 和 setInterval計時器 clearInterval()清除計時器 效果如下: 實作代碼如下: <!DOCTYPE html> <html lang="en"> <head> <meta charset="UTF-8"> <meta ......

    uj5u.com 2023-04-20 08:12:31 more
  • JavaScript 運算子

    JavaScript 運算子/運算子 在 JavaScript 中,有一些運算子可以使代碼更簡潔、易讀和高效。以下是一些常見的運算子: 1、可選鏈運算子(optional chaining operator) ?.是可選鏈運算子(optional chaining operator)。?. 可選鏈操 ......

    uj5u.com 2023-04-20 08:02:25 more
  • CSS—相對單位rem

    一、概述 rem是一個相對長度單位,它的單位長度取決于根標簽html的字體尺寸。rem即root em的意思,中文翻譯為根em。瀏覽器的文本尺寸一般默認為16px,即默認情況下: 1rem = 16px rem布局原理:根據CSS媒體查詢功能,更改根標簽的字體尺寸,實作rem單位隨螢屏尺寸的變化,如 ......

    uj5u.com 2023-04-20 08:02:21 more
  • 我的第一個NPM包:panghu-planebattle-esm(胖虎飛機大戰)使用說明

    好家伙,我的包終于開發完啦 歡迎使用胖虎的飛機大戰包!! 為你的主頁添加色彩 這是一個有趣的網頁小游戲包,使用canvas和js開發 使用ES6模塊化開發 效果圖如下: (覺得圖片太sb的可以自己改) 代碼已開源!! Git: https://gitee.com/tang-and-han-dynas ......

    uj5u.com 2023-04-20 08:01:50 more
  • 如何在 vue3 中使用 jsx/tsx?

    我們都知道,通常情況下我們使用 vue 大多都是用的 SFC(Signle File Component)單檔案組件模式,即一個組件就是一個檔案,但其實 Vue 也是支持使用 JSX 來撰寫組件的。這里不討論 SFC 和 JSX 的好壞,這個仁者見仁智者見智。本篇文章旨在帶領大家快速了解和使用 Vu ......

    uj5u.com 2023-04-20 08:01:37 more
  • 【Vue2.x原始碼系列06】計算屬性computed原理

    本章目標:計算屬性是如何實作的?計算屬性快取原理以及洋蔥模型的應用?在初始化Vue實體時,我們會給每個計算屬性都創建一個對應watcher,我們稱之為計算屬性watcher ......

    uj5u.com 2023-04-20 08:01:31 more
  • http1.1與http2.0

    一、http是什么 通俗來講,http就是計算機通過網路進行通信的規則,是一個基于請求與回應,無狀態的,應用層協議。常用于TCP/IP協議傳輸資料。目前任何終端之間任何一種通信方式都必須按Http協議進行,否則無法連接。tcp(三次握手,四次揮手)。 請求與回應:客戶端請求、服務端回應資料。 無狀態 ......

    uj5u.com 2023-04-20 08:01:10 more
  • http1.1與http2.0

    一、http是什么 通俗來講,http就是計算機通過網路進行通信的規則,是一個基于請求與回應,無狀態的,應用層協議。常用于TCP/IP協議傳輸資料。目前任何終端之間任何一種通信方式都必須按Http協議進行,否則無法連接。tcp(三次握手,四次揮手)。 請求與回應:客戶端請求、服務端回應資料。 無狀態 ......

    uj5u.com 2023-04-20 08:00:32 more